@@ -56,7 +56,7 @@ class BaseModalMixin(AjaxHelpers):
         else:
             context = {}
         context.update({'request': self.request, 'slug': self.slug})
-        context['modal_url'] = kwargs.get('modal_url', self.request.path)
+        context['modal_url'] = kwargs.get('modal_url', self.request.get_full_path())
         context['no_header_x'] = getattr(self, 'no_header_x', None)
         context['center_header'] = kwargs.get('center_header', getattr(self, 'center_header', None))
         context['size'] = kwargs.get('size', self.size)
@@ -85,7 +85,7 @@ class BaseModalMixin(AjaxHelpers):
             raise ModalException('User does not have permission')
 
     def button_refresh_modal(self, **_kwargs):
-        return self.command_response('')
+        return self.command_response(ajax_modal_replace(self.request, modal_class=self.__class__))
 
     def button_group(self):
         button_kwargs = {
@@ -186,10 +186,21 @@ class FormModalMixin(BaseModalMixin):
             return HttpResponse(render_crispy_form(form))
         return self.refresh_form(form)
 
+    def create_object(self):
+        pass
+
+    def update_object(self):
+        pass
+
     def form_valid(self, form):
+        org_id = self.object.id
         save_function = getattr(form, 'save', None)
         if save_function:
             save_function()
+        if org_id:
+            self.update_object()
+        else:
+            self.create_object()
         if not self.response_commands:
             self.add_command('reload')
         return self.command_response()
@@ -240,8 +251,8 @@ class FormModalMixin(BaseModalMixin):
                                      html=render_to_string(self.template_name, self.get_context_data()))
 
     def button_refresh_modal(self, **kwargs):
-        if self.slug.get('readonly'):
-            return ''
+        if self.slug.get('readonly') or kwargs.get('whole_modal'):
+            return super().button_refresh_modal()
         else:
             form = self.get_form()
             form.clear_errors()
@@ -352,9 +363,13 @@ class ModelFormModal(SingleObjectMixin, FormModal):
         kwargs.update(self.form_init_args)
         return kwargs
 
+    def object_delete(self):
+        pass
+
     def button_confirm_delete(self, **_kwargs):
         if self.process in [PROCESS_DELETE, PROCESS_EDIT_DELETE]:
             self.object.delete()
+        self.object_delete()
         if not self.response_commands:
             self.add_command('close', no_refresh=True)
             self.add_command('reload')
